# Installation

Run `composer require require dedi/sylius-voucher-plugin`

Add plugin class to your `config/bundles.php`:

Make sure you add it before SyliusGridBundle, otherwise you'll get You have requested a non-existent parameter "dedi_sylius_voucher.model.voucher.class". exception.

```php
<?php
$bundles = [
    // ...
    Dedi\SyliusVoucherPlugin\DediSyliusVoucherPlugin::class => ['all' => true],
    Sylius\Bundle\GridBundle\SyliusGridBundle::class => ['all' => true],
    // ...
];
```

# Basic Configuration

Import configuration:

```yaml
# config/packages/dedi_sylius_voucher.yaml

imports:
    # 
    - { resource: "@DediSyliusVoucherPlugin/Resources/config/app/config.yaml" }
```

(Optional) Import fixtures

If you wish to have some vouchers to play with in your application during development.

```yaml
# config/packages/dedi_sylius_voucher.yaml
imports:
    # ...
    - { resource: "@DediSyliusVoucherPlugin/Resources/config/app/fixtures.yaml" }
```

Import routing:

```yaml
# config/routes.yaml
dedi_sylius_voucher:
    resource: "@DediSyliusVoucherPlugin/Resources/config/routes.yaml"
```
or if your app doesn't use locales:

```yaml
# config/routes.yaml
dedi_sylius_voucher:
    resource: "@DediSyliusVoucherPlugin/Resources/config/routes_no_locale.yaml"
```

Override templates:

```shell
cp -R vendor/dedi/sylius-voucher-plugin/tests/Application/templates/* templates
```


Extend Product

```php
<?php

# src/Entity/Product/Product.php

declare(strict_types=1);
namespace App\Entity\Product;

use Doctrine\ORM\Mapping as ORM;
use Dedi\SyliusVoucherPlugin\Model\ProductInterface as DediSyliusvoucherProductInterface;
use Dedi\SyliusVoucherPlugin\Model\ProductTrait as DediSyliusvoucherProductTrait;
use Sylius\Component\Core\Model\Product as BaseProduct;

/**
 * @ORM\Entity
 * @ORM\Table(name="sylius_product")
 */
class Product extends BaseProduct implements DediSyliusvoucherProductInterface
{
    use DediSyliusvoucherProductTrait;
}
```

Extend Order

```php
<?php

# src/Entity/Order/Order.php

declare(strict_types=1);
namespace App\Entity\Order;

use Dedi\SyliusVoucherPlugin\Model\OrderInterface as DediSyliusVoucherPluginOrderInterface;
use Dedi\SyliusVoucherPlugin\Model\OrderTrait as DediSyliusVoucherPluginOrderTrait;
use Sylius\Component\Core\Model\Order as BaseOrder;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity
 * @ORM\Table(name="sylius_order")
 */
class Order extends BaseOrder implements DediSyliusVoucherPluginOrderInterface
{
    use DediSyliusVoucherPluginOrderTrait {
        DediSyliusVoucherPluginOrderTrait::__construct as private __voucherTraitConstruct;
    }
    
    public function __construct()
    {
        $this->__voucherTraitConstruct();

        parent::__construct();
    }
}
```

Extend OrderRepository:

```php
<?php

# src/Doctrine/ORM/OrderRepository.php

declare(strict_types=1);
namespace App\Doctrine\ORM;

use Dedi\SyliusVoucherPlugin\Repository\OrderRepositoryInterface as DediSyliusVoucherPluginOrderRepositoryInterface;
use Dedi\SyliusVoucherPlugin\Doctrine\ORM\OrderRepositoryTrait as DediSyliusVoucherPluginOrderRepositoryTrait;
use Sylius\Bundle\CoreBundle\Doctrine\ORM\OrderRepository as BaseOrderRepository;

class OrderRepository extends BaseOrderRepository implements DediSyliusVoucherPluginOrderRepositoryInterface
{
    use DediSyliusVoucherPluginOrderRepositoryTrait;
}
```

Extend CustomerRepository:

```php
<?php

# src/Doctrine/ORM/CustomerRepository.php

declare(strict_types=1);
namespace App\Doctrine\ORM;

use Dedi\SyliusVoucherPlugin\Repository\CustomerRepositoryInterface as DediSyliusVoucherPluginCustomerRepositoryInterface;
use Dedi\SyliusVoucherPlugin\Doctrine\ORM\CustomerRepositoryTrait as DediSyliusVoucherPluginCustomerRepositoryTrait;
use Sylius\Bundle\CoreBundle\Doctrine\ORM\CustomerRepository as BaseCustomerRepository;

class CustomerRepository extends BaseCustomerRepository implements DediSyliusVoucherPluginCustomerRepositoryInterface
{
    use DediSyliusVoucherPluginCustomerRepositoryTrait;
}
```

Add configuration:

```yaml
# config/services.yaml
sylius_customer:
    resources:
       customer:
            classes:
                repository: App\Doctrine\ORM\CustomerRepository

sylius_order:
    resources:
        order:
            classes:
                model: App\Entity\Order\Order
                repository: App\Doctrine\ORM\OrderRepository
                
sylius_product:
    resources:
        product:
            classes:
                model: App\Entity\Product\Product
```


## Avance configuration

```yaml
dedi_sylius_voucher:
    use_no_restriction_rule: true # See Custom rules section
```

## Custom rules

You may want to apply some rules to make some vouchers available for a given order in the checkout process. By default, a rule allowing every voucher for every order (except vouchers applied on the given order) is defined. If you want to add your custom rules, this rule should be disabled. It can be done by defining `dedi_sylius_voucher.use_no_restriction_rule` configuration value to false. See [configuration](#configure-the-plugin).

In order to implement custom rules, you should implement `VoucherRuleInterface`. Services implementing this interface will programmatically being tagged `dedi_sylius_voucher.eligibility.rule`.
