<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Checker;

use Exception;
use Psr\Log\LoggerInterface;
use Sylius\Component\Order\Context\CartContextInterface;
use Sylius\Component\Order\Model\OrderInterface;

final class VoucherCheckerContext implements VoucherCheckerContextInterface
{
    private readonly LoggerInterface $logger;

    private readonly VoucherFilterInterface $voucherFilter;

    private readonly CartContextInterface $cartContext;

    private $rules;

    public function __construct(LoggerInterface $logger, VoucherFilterInterface $voucherFilter, CartContextInterface $cartContext, iterable $rules)
    {
        $this->voucherFilter = $voucherFilter;
        $this->cartContext = $cartContext;
        $this->rules = $rules;
        $this->logger = $logger;
    }

    public function getVouchersEligible(array $vouchers): array
    {
        try {
            /** @var OrderInterface $order */
            $order = $this->cartContext->getCart();

            return $this->voucherFilter->filterEligibleVoucher(
                $order,
                $this->rules,
                $vouchers,
            );
        } catch (Exception $e) {
            $this->logger->error($e->getMessage());
        }

        return [];
    }
}
