<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Controller\Action;

use Dedi\SyliusVoucherPlugin\Applicator\VoucherApplicatorInterface;
use Dedi\SyliusVoucherPlugin\Form\Type\AddVoucherToOrderType;
use Dedi\SyliusVoucherPlugin\Model\OrderInterface;
use FOS\RestBundle\View\View;
use FOS\RestBundle\View\ViewHandlerInterface;
use Sylius\Component\Order\Context\CartContextInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Session\Flash\FlashBagInterface;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Webmozart\Assert\Assert;

final class AddVoucherToOrderAction extends AbstractController
{
    private readonly ViewHandlerInterface $viewHandler;

    private readonly FormFactoryInterface $formFactory;

    private readonly CartContextInterface $cartContext;

    private readonly FlashBagInterface $flashBag;

    private readonly UrlGeneratorInterface $urlGenerator;

    private readonly VoucherApplicatorInterface $voucherApplicator;

    public function __construct(
        ViewHandlerInterface $viewHandler,
        FormFactoryInterface $formFactory,
        CartContextInterface $cartContext,
        FlashBagInterface $flashBag,
        UrlGeneratorInterface $urlGenerator,
        VoucherApplicatorInterface $voucherApplicator,
    ) {
        $this->viewHandler = $viewHandler;
        $this->formFactory = $formFactory;
        $this->cartContext = $cartContext;
        $this->flashBag = $flashBag;
        $this->urlGenerator = $urlGenerator;
        $this->voucherApplicator = $voucherApplicator;
    }

    public function __invoke(Request $request): Response
    {
        /** @var OrderInterface|null $order */
        $order = $this->cartContext->getCart();
        Assert::notNull($order);

        $addVoucherToOrderCommand = new AddVoucherToOrderCommand();
        $form = $this->formFactory->create(AddVoucherToOrderType::class, $addVoucherToOrderCommand);
        $form->handleRequest($request);

        if ($form->isSubmitted()) {
            if ($form->isValid()) {
                $this->voucherApplicator->apply($order, $addVoucherToOrderCommand->getVoucher());

                $this->flashBag->add('success', 'dedi_sylius_voucher.voucher_added');

                if ($request->isXmlHttpRequest()) {
                    return $this->viewHandler->handle(View::create([], Response::HTTP_CREATED));
                }

                return new RedirectResponse($this->urlGenerator->generate('sylius_shop_checkout_select_payment'));
            }

            if ($request->isXmlHttpRequest()) {
                return $this->viewHandler->handle(View::create($form, Response::HTTP_BAD_REQUEST)->setData([
                    'errors' => $form->getErrors(true, true),
                ]));
            }

            $this->flashBag->add('error', 'dedi_sylius_voucher.voucher_added_error');

            return new RedirectResponse($this->urlGenerator->generate('sylius_shop_checkout_select_payment'));
        }

        return $this->render('@DediSyliusVoucherPlugin/Shop/addVoucherToOrder.html.twig', ['form' => $form->createView()]);
    }
}
