<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Form\DataTransformer;

use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Symfony\Component\Form\DataTransformerInterface;
use Symfony\Component\Form\Exception\TransformationFailedException;
use Webmozart\Assert\Assert;

final class VoucherToCodeDataTransformer implements DataTransformerInterface
{
    private readonly VoucherRepositoryInterface $voucherRepository;

    private readonly ChannelContextInterface $channelContext;

    public function __construct(
        VoucherRepositoryInterface $voucherRepository,
        ChannelContextInterface $channelContext,
    ) {
        $this->voucherRepository = $voucherRepository;
        $this->channelContext = $channelContext;
    }

    /**
     * @param VoucherInterface|mixed $value
     */
    public function transform($value): ?string
    {
        if (null === $value || '' === $value) {
            return $value;
        }

        Assert::isInstanceOf($value, VoucherInterface::class);

        return $value->getCode();
    }

    public function reverseTransform($value): ?VoucherInterface
    {
        if (null === $value || '' === $value) {
            return null;
        }

        $voucher = $this->voucherRepository->findOneEnabledNotExpiredByCodeAndChannel(
            $value,
            $this->channelContext->getChannel(),
        );

        if (null !== $voucher) {
            return $voucher;
        }

        throw new TransformationFailedException('dedi_sylius_voucher.ui.voucher_code_does_not_exist');
    }
}
