<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\OrderProcessor;

use Dedi\SyliusVoucherPlugin\Model\AdjustmentInterface;
use Dedi\SyliusVoucherPlugin\Model\OrderInterface;
use Sylius\Component\Order\Factory\AdjustmentFactoryInterface;
use Sylius\Component\Order\Model\OrderInterface as BaseOrderInterface;
use Sylius\Component\Order\Processor\OrderProcessorInterface;
use Symfony\Contracts\Translation\TranslatorInterface;
use Webmozart\Assert\Assert;

final class OrderVoucherProcessor implements OrderProcessorInterface
{
    private readonly TranslatorInterface $translator;

    private readonly AdjustmentFactoryInterface $adjustmentFactory;

    public function __construct(
        TranslatorInterface $translator,
        AdjustmentFactoryInterface $adjustmentFactory,
    ) {
        $this->translator = $translator;
        $this->adjustmentFactory = $adjustmentFactory;
    }

    /**
     * @param BaseOrderInterface|OrderInterface $order
     */
    public function process(BaseOrderInterface $order): void
    {
        Assert::isInstanceOf($order, OrderInterface::class);

        if ($order->isEmpty()) {
            return;
        }

        if (!$order->hasVouchers()) {
            return;
        }

        foreach ($order->getVouchers() as $voucher) {
            $amount = $voucher->getAmount();
            $total = $order->getTotal();

            if ($total < $amount) {
                $amount = $total;
            }

            if (0 === $amount) {
                continue;
            }

            $adjustment = $this->adjustmentFactory->createWithData(
                AdjustmentInterface::ORDER_VOUCHER_ADJUSTMENT,
                $this->translator->trans('dedi_sylius_voucher.ui.voucher'),
                -1 * $amount,
            );

            $adjustment->setOriginCode($voucher->getCode());

            $order->addAdjustment($adjustment);
        }
    }
}
