<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Security;

use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use LogicException;
use Sylius\Component\Core\Model\AdminUserInterface;
use Sylius\Component\Core\Model\ShopUserInterface;
use Sylius\Component\User\Model\UserInterface;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authorization\Voter\Voter;
use Webmozart\Assert\Assert;

final class VoucherVoter extends Voter
{
    public const READ = 'read';

    protected function supports($attribute, $subject): bool
    {
        if (!\in_array($attribute, [self::READ], true)) {
            return false;
        }

        if (!$subject instanceof VoucherInterface) {
            return false;
        }

        return true;
    }

    protected function voteOnAttribute($attribute, $subject, TokenInterface $token): bool
    {
        /** @var UserInterface|ShopUserInterface|AdminUserInterface|null $user */
        $user = $token->getUser();
        if (!$user instanceof UserInterface) {
            return false;
        }

        // Admin can do anything with vouchers
        if ($user instanceof AdminUserInterface) {
            return true;
        }
        Assert::isInstanceOf($user, ShopUserInterface::class);

        /** @var VoucherInterface $voucher */
        $voucher = $subject;
        switch ($attribute) {
            case self::READ:
                return $this->canRead($voucher, $user);
        }

        throw new LogicException('This code should not be reached.');
    }

    private function canRead(VoucherInterface $voucher, ShopUserInterface $user): bool
    {
        // Anonymous vouchers can be seen by everyone
        if (null === $voucher->getCustomer()) {
            return true;
        }

        // Vouchers can only be seen by their proprietary
        return $voucher->getCustomer() === $user->getCustomer();
    }
}
